#include "virtual_can.h"
#include "commexceptions.h"
#include <string.h>
#include <iostream>
#include <math.h>
#include <errno.h>
#include <sys/time.h>
#include <stdlib.h>     /* srand, rand */
#include <time.h>       /* time */
#include <iomanip>

CVirtualCAN::CVirtualCAN(const std::string &comm_id) : CComm(comm_id)
{
  this->dump_filename="";
  this->req_can_id=-1;
  this->rx_filters=NULL;
  this->num_filters=0;
  srand(time(NULL));
  // initialize the thread attributes
  this->thread_server=CThreadServer::instance();
  this->can_thread_id=comm_id + "_can_thread";
  this->thread_server->create_thread(this->can_thread_id);
  this->thread_server->attach_thread(this->can_thread_id,this->can_thread,this);
  // initialize the event atributes
  this->event_server=CEventServer::instance();
  this->new_frame_event_id=comm_id + "_new_frame_event_id";
  this->event_server->create_event(this->new_frame_event_id);
  this->frame_error_event_id=comm_id + "_frame_error_event_id";
  this->event_server->create_event(this->frame_error_event_id);
  this->data_requested_event_id=comm_id + "_data_requested_event_id";
  this->event_server->create_event(this->data_requested_event_id);
  this->finish_can_thread_event_id=comm_id + "_finish_can_thread_event_id";
  this->event_server->create_event(this->finish_can_thread_event_id);
  this->update_rate_hz=1000;
  this->err_rate = 0;
}

void CVirtualCAN::hard_open(void *comm_dev)
{
  std::string *dump_filename=(std::string *)comm_dev;

  this->dump_file.exceptions(std::ifstream::failbit|std::ifstream::eofbit);// enable exceptions
  try{
    this->dump_file.open(dump_filename->c_str(),std::ifstream::in);// try to open the file for reading
    this->dump_filename=*dump_filename;
  }catch(std::ifstream::failure &e){
    throw CCommException(_HERE_,"Impossible to open the desired dump file",this->comm_id);
  }
}

void CVirtualCAN::hard_config(void *config)
{
  // start the can thread
  this->thread_server->start_thread(this->can_thread_id);
}

int CVirtualCAN::hard_read(unsigned char *data, int len)
{
  struct can_frame *frame=(struct can_frame *)data; 
  char line[1024];
  bool valid_id=true;
  unsigned int i;

  do{
    valid_id=true;
    // read a line from the file
    try{
      this->dump_file.getline(line,1024);
    }catch(std::ifstream::failure &e){
      /* close and reopen the file */
      this->dump_file.close();
      this->dump_file.open(this->dump_filename.c_str(),std::ifstream::in);
      this->dump_file.getline(line,1024);
    }
    if((sscanf(line,"  slcan0  %x   [%hhd]  %hhx %hhx %hhx %hhx %hhx %hhx %hhx %hhx",&frame->can_id,&frame->can_dlc,&frame->data[0],&frame->data[1],&frame->data[2],
              &frame->data[3],&frame->data[4],&frame->data[5],&frame->data[6],&frame->data[7]))!=10)
    {
      throw CCommException(_HERE_,"Invalid file format.",this->comm_id);
    }
    // filter the incomming packet
    for(i=0;i<this->num_filters;i++)
    {
      if((frame->can_id&this->rx_filters[i].can_mask)!=this->rx_filters[i].can_id)
        valid_id=false;
    }
  }while(!valid_id);

  return len;
}

int CVirtualCAN::hard_write(unsigned char *data, int len)
{
  return len;
}

int CVirtualCAN::hard_get_num_data(void)
{
  return sizeof(struct can_frame);// return the length of a full packet
}

int CVirtualCAN::hard_wait_comm_event(void)
{
  usleep(1000000/this->update_rate_hz);// publish can packets at 1KHz
  return 1;
}

void CVirtualCAN::hard_close(void)
{
  this->dump_file.close();
}

void *CVirtualCAN::can_thread(void *param)
{
  struct can_frame new_frame;
  std::list<std::string> events;
  CVirtualCAN *can=(CVirtualCAN *)param;
  unsigned int event_id,num;
  bool end=false;

  events.push_back(can->finish_can_thread_event_id);
  events.push_back(can->get_rx_event_id());
  while(!end)
  {
    event_id=can->event_server->wait_first(events);
    if(event_id==0)
      end=true;
    else
    {
      can->can_access.enter();
      // process the incomming data
      num=can->get_num_data();
      while(num>=sizeof(struct can_frame))// there is a whole frame in the input buffer
      {
        can->read((unsigned char *)&new_frame,sizeof(struct can_frame));
        if(new_frame.can_id&CAN_RTR_FLAG)
        {
          // save the requested can_id
          can->req_can_id=new_frame.can_id;
          // activate the event
          can->event_server->set_event(can->data_requested_event_id);
        }
        else
        {
          //puch the new frame into the queue
          // activate the event
          if ((rand() % 100 + 1) > can->err_rate)
          {
            can->rx_frames.push(new_frame);
            can->event_server->set_event(can->new_frame_event_id);
          }
          else
          {
            std::cout << "lost : " << std::uppercase << std::hex << (int) new_frame.can_id << "  ";
            for (int ii = 0; ii < new_frame.can_dlc; ii++)
              std::cout << std::uppercase << std::setw(2) << std::setfill('0') << std::hex << std::hex << (int) new_frame.data[ii] << "  ";
            std::cout << std::dec << std::endl;
          }
        }
        num-=sizeof(struct can_frame);
      }
      can->can_access.exit();
    }
  }
 
  pthread_exit(NULL);
}

void CVirtualCAN::open(const std::string &dump_filename)
{
  CComm::open((void *)&dump_filename);
  this->config();
}

void CVirtualCAN::set_update_rate(double rate_hz)
{
  if(rate_hz<10.0 || rate_hz>10000.0)
    throw CCommException(_HERE_,"Invalid update rate. Supported rate range between 10 Hz and 10 kHz.",this->comm_id);
  else
    this->update_rate_hz=rate_hz;
}

double CVirtualCAN::get_update_rate(void)
{
  return this->update_rate_hz;
}

std::string CVirtualCAN::get_new_frame_event_id(void)
{
  return this->new_frame_event_id;
}

std::string CVirtualCAN::get_frame_error_event_id(void)
{
  return this->frame_error_event_id;
}

std::string CVirtualCAN::get_data_requested_event_id(void)
{
  return this->data_requested_event_id;
}

unsigned int CVirtualCAN::get_requested_can_id(void)
{ 
  return this->req_can_id;
}

unsigned long CVirtualCAN::get_last_rx_timestamp(void)
{ 
  struct timeval tv;
  
  gettimeofday (&tv, NULL);

  return tv.tv_sec*1000000+tv.tv_usec;
}

void CVirtualCAN::write_frame(unsigned int can_id, unsigned char *data, int len)
{
  int num_frames=ceil((double)len/8.0),i,j;
  struct can_frame frame;

  // check the identifier format
  if((can_id&(~CAN_SFF_MASK))!=0)// it's an extended identifier
    can_id=can_id|CAN_EFF_FLAG;

  for(i=0;i<num_frames;i++)
  {
    // generate all the frames
    frame.can_id=can_id;
    if((len-8*i)<8)
      frame.can_dlc=len-8*i;
    else
      frame.can_dlc=8;
    for(j=0;j<frame.can_dlc;j++)
      frame.data[j]=data[i*8+j];
    // send the frames
    this->write((unsigned char *)&frame,sizeof(struct can_frame));
  }
}

void CVirtualCAN::read_frame(unsigned int *can_id, unsigned char *data, int *len)
{
  struct can_frame frame;
  int i=0;

  this->can_access.enter();
  if(!this->rx_frames.empty())
  {
    frame=this->rx_frames.front();
    this->rx_frames.pop();
    *can_id=frame.can_id;
    *len=frame.can_dlc; //COMMENT: Do we need a cast ? frame.can_dlc is __u8 type variable (see /usr/include/linux/can.h) 
    for(i=0;i<frame.can_dlc;i++)
      data[i]=frame.data[i];
  }
  else
  {
    /* handle exceptions */
    this->can_access.exit();    
    throw CCommException(_HERE_,"No data available",this->comm_id);
  }
  this->can_access.exit();
}

void CVirtualCAN::add_id_filter(unsigned short int can_id,bool invert)
{
  struct can_filter *new_filters;

  new_filters=new struct can_filter[this->num_filters+1];
  memcpy(new_filters,this->rx_filters,sizeof(struct can_filter)*this->num_filters);
  if(this->rx_filters!=NULL)
    delete[] this->rx_filters;
  this->rx_filters=new_filters;
  // update the new filter
  if((can_id&(~CAN_SFF_MASK))!=0)// it's an extended identifier
  {
    this->rx_filters[this->num_filters].can_id=can_id|CAN_EFF_FLAG;
    this->rx_filters[this->num_filters].can_mask=CAN_EFF_MASK;
  }
  else
  {
    this->rx_filters[this->num_filters].can_id=can_id;
    this->rx_filters[this->num_filters].can_mask=CAN_SFF_MASK;
  }
  if(invert)
    this->rx_filters[this->num_filters].can_id|=CAN_INV_FILTER;
  // set up the new filters
  this->num_filters++;
}

void CVirtualCAN::add_id_filter(unsigned short int can_id,unsigned short int mask,bool invert)
{
  struct can_filter *new_filters;

  new_filters=new struct can_filter[this->num_filters+1];
  memcpy(new_filters,this->rx_filters,sizeof(struct can_filter)*this->num_filters);
  if(this->rx_filters!=NULL)
    delete[] this->rx_filters;
  this->rx_filters=new_filters;
  // update the new filter
  if((can_id&(~CAN_SFF_MASK))!=0)// it's an extended identifier
  {
    this->rx_filters[this->num_filters].can_id=can_id|CAN_EFF_FLAG;
    this->rx_filters[this->num_filters].can_mask=mask;
  }
  else
  {
    this->rx_filters[this->num_filters].can_id=can_id;
    this->rx_filters[this->num_filters].can_mask=mask;
  }
  if(invert)
    this->rx_filters[this->num_filters].can_id|=CAN_INV_FILTER;
  // set up the new filters
  this->num_filters++;
}

void CVirtualCAN::clear_id_filters(void)
{
  if(this->rx_filters!=NULL)
  {
    delete this->rx_filters;
    this->rx_filters=NULL;
    this->num_filters=0;
  }
}
void CVirtualCAN::set_error_rate(double err_rate)
{
  if (err_rate >= 0 && err_rate <= 100)
  {
    this->can_access.enter();
    this->err_rate = err_rate;
    this->can_access.exit();
  }
  else
    throw CCommException(_HERE_,"Invalid error rate. It must be a value between 0 and 100",this->comm_id);
}

double CVirtualCAN::get_error_rate(void)
{
  double err_rate;
  this->can_access.enter();
  err_rate = this->err_rate;
  this->can_access.exit();
  return err_rate;
}

void CVirtualCAN::close(void)
{
  CComm::close();
}

CVirtualCAN::~CVirtualCAN()
{
  // close the can device
  CComm::close();
  // free all the resources
  if(this->finish_can_thread_event_id!="")
  {
    this->event_server->set_event(this->finish_can_thread_event_id);
    this->thread_server->end_thread(this->can_thread_id);
    this->thread_server->detach_thread(this->can_thread_id);
    this->thread_server->delete_thread(this->can_thread_id);
    this->can_thread_id="";
    this->event_server->delete_event(this->finish_can_thread_event_id);
    this->finish_can_thread_event_id="";
  }
  if(this->new_frame_event_id!="")
  {
    this->event_server->delete_event(this->new_frame_event_id);
    this->new_frame_event_id="";
  }
  if(this->frame_error_event_id!="")
  {
    this->event_server->delete_event(this->frame_error_event_id);
    this->frame_error_event_id="";
  }
  if(this->data_requested_event_id!="")
  {
    this->event_server->delete_event(this->data_requested_event_id);
    this->data_requested_event_id="";
  }
}

